import javax.microedition.lcdui.*;
import javax.microedition.rms.*;
import java.util.*;
import java.io.*;
import menu.*;
import store.*;

/**
* An AccountIndexPage holds lists of AccountPages.  The
* user selects an AccountPage from the list for editing.
* This class extends MenuPage and adds the ability to 
* recognize when the RMS store has changed and then
* rebuild the list of AccountPages.
*/
public class AccountIndexPage extends MenuPage {
	/**
	* Initialize a new AccountIndexPage object using the supplied parameters.
	* @param label short label for this page (as might be used in
	* a menu)
	* @param title longer label for this page (as might be used in
	* a screen title)
	* @param icon small icon for this page (as might be used in
	* a toolbar or menu
	* @param hasExit true if this page should have an Exit command.
	*/
	public AccountIndexPage(String label,String title,Image icon,boolean hasExit) {
		super(label,title,icon,hasExit);
	}
	/**
	* Initialize a new AccountIndexPage object using the supplied parameters
	* and default values where needed.
	* The title defaults to null (meaning that the label will be used
	* in its place).  
	* The icon defaults to null (meaning text only will be displayed if
	* this page is shown in another MenuPage).
	* @param label short label for this page (as might be used in
	* a menu)
	* @param hasExit true if this page should have an Exit command.
	*/
	public AccountIndexPage(String label,boolean hasExit) {
		super(label,null,null,hasExit);
	}
	/**
	* Return the Displayable object for this AccountIndexPage, 
	* completely filled in and ready to display.  If the RecordStore
	* has changed since we last built the list of AccountPages,
	* then we rebuild the list.
	* @return the appropriate Displayable object for this Page
	*/
	public Displayable getDisplayable() {
		try {
			RecordStore store = RecordStore.openRecordStore(ACCOUNT_STORE,true);
			int newVersion = store.getVersion();
			
			if ((!initialized) || (initialized && newVersion != currentVersion)) {
				this.removeAllChildren();
				UserAccount newAccount = new UserAccount();
				AccountPage newAccountPage = 
				new AccountPage("<new>",new StorableObject(ACCOUNT_STORE,0,newAccount));
				this.addChild(newAccountPage);
				
				RecordEnumeration re = store.enumerateRecords(null,null,false);
				while(re.hasNextElement()) {
					int id = re.nextRecordId();
					UserAccount usr = new UserAccount();
					StorableObject sObj = new StorableObject(ACCOUNT_STORE,id,usr);
					sObj.readUserDataObject();
					this.addChild(new AccountPage(usr.getLabel(),sObj));
				}
				currentVersion = newVersion;
				initialized = true;
			}
			store.closeRecordStore();
		}
		catch (RecordStoreException e) {
			e.printStackTrace();				// this should be an alert ...
		}
		catch (IOException e) {
			e.printStackTrace();				// this should be an alert ...
		}
		return super.getDisplayable();
	}
	
	//----------------------------------------------------------------------
	
	/** false if we have never built a list of AccountPage children */
	private boolean initialized = false;
	/** 
	* if we have built a list of AccountPages, this is the version of the
	* RecordStore that was in use at the time.
	*/
	private int currentVersion;
		
	//--------------------------------------------------------------------
	
	/** the name of the RecordStore that we are keeping accounts in */
	public static final String ACCOUNT_STORE = "Accounts";

}
